package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.pikamug.quests.quests.Quest;
import me.pikamug.quests.quests.components.Rewards;
import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.StringReplacer;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.List;

import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_COMMAND;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_EXPERIENCE;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_ITEM;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_MONEY;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_PERMISSION;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REWARDS_QUEST_POINTS;

/**
 * A class to generate a string containing all information about a quest's rewards
 */
public class QuestRewardsInfoGenerator {

    private final Quest quest;
    private final StringFormatter formatter;

    /**
     * Instantiates a new quest reward info generator
     *
     * @param quest <p>The quest to generate information about</p>
     */
    public QuestRewardsInfoGenerator(@NotNull Quest quest) {
        this.quest = quest;
        this.formatter = DynmapCitizens.getFormatter();
    }

    /**
     * Gets information about all rewards for the given quest
     *
     * @return <p>Information about the quest's rewards</p>
     */
    public @NotNull String getQuestRewardsInfo() {
        Rewards reward = quest.getRewards();
        StringReplacer replacer = new StringReplacer(formatter.getUnFormattedColoredMessage(QUESTS_REWARDS_FORMAT));

        replacer.add("{money}", reward.getMoney() > 0 ? formatter.replacePlaceholders(QUESTS_REWARDS_MONEY,
                List.of("{amount}", "{currency}"), List.of(String.valueOf(reward.getMoney()),
                        QuestsHelper.getCurrency(reward.getMoney()))) : "");

        replacer.add("{experience}", reward.getExp() > 0 ? formatter.replacePlaceholder(
                QUESTS_REWARDS_EXPERIENCE, "{amount}", String.valueOf(reward.getExp())) : "");

        replacer.add("{questPoints}", reward.getQuestPoints() > 0 ? formatter.replacePlaceholder(
                QUESTS_REWARDS_QUEST_POINTS, "{amount}", String.valueOf(reward.getQuestPoints())) : "");

        StringBuilder permissionBuilder = new StringBuilder();
        for (String permission : reward.getPermissions()) {
            permissionBuilder.append(formatter.replacePlaceholder(QUESTS_REWARDS_PERMISSION, "{permission}",
                    permission));
        }
        replacer.add("{permissions}", permissionBuilder.toString());

        StringBuilder itemBuilder = new StringBuilder();
        for (Object item : reward.getItems()) {
            itemBuilder.append(formatter.replacePlaceholder(QUESTS_REWARDS_ITEM, "{item}",
                    QuestsHelper.getUpperCasedItemStackString((ItemStack) item)));
        }
        replacer.add("{items}", itemBuilder.toString());

        StringBuilder commandBuilder = new StringBuilder();
        for (String command : reward.getCommands()) {
            commandBuilder.append(formatter.replacePlaceholder(QUESTS_REWARDS_COMMAND, "{command}", command));
        }
        replacer.add("{commands}", commandBuilder.toString());

        return replacer.replace();
    }

}
